function [dualUpper,L,U] = derivedualBounds(H,c,A,b,E,f,ops,parametricDomain)

if isempty(A)
    dualUpper = [];
    L = [];
    U = [];
    return
end

n = length(c);
m = length(b);

x = sdpvar(n,1);
Lambda = sdpvar(m,1);

F = A*x < b;

% Start by computing primal lower bounds
if nargin < 7
    ops = sdpsettings('verbose',0);
end
ops2 = ops;
ops2.verbose = max(0,ops.verbose-1);;
all_bounded = 1;
if ops.verbose
    disp(['*Computing ' num2str(length(x)) ' primal bounds (required for dual bounds)']);
end

z = recover(depends(c));
xz = [x;z];
nz = length(z);
nTOT = n + length(z);
sol = solvesdp([F,parametricDomain,H*x + c + A'*Lambda==0,Lambda>0],[xz;-xz],ops2);
for i = 1:nTOT
    if sol.problem(i)==0
        selectsolution(i);
        L(i,1) = double(xz(i));
    else
        L(i,1) = inf;
    end
    if sol.problem(nTOT+i)==0
        selectsolution(nTOT+i);
        U(i,1) = double(xz(i));
    else
        U(i,1) = inf;
    end
end

c0C = full(getbase(c));
c0 = c0C(:,1);
C = c0C(:,2:end);

b0B = full(getbase(b));
b0 = b0B(:,1);
B = b0B(:,2:end);

if isa(b,'sdpvar')
    if ops.verbose
        disp(['*Computing ' num2str(length(b)) ' bounds on parameterized RHS (required for dual bounds)']);
    end
    bmax = inf(length(b),1);
    bmin = inf(length(b),1);
    sol = solvesdp([F,parametricDomain],[-b;b],ops2);
    for i = 1:length(b)
        if sol.problem(i) == 0
            selectsolution(i);
            bmax(i) = double(b(i));
        end
        if sol.problem(i+length(b)) == 0
            selectsolution(i+length(b));
            bmin(i) = double(b(i));
        end
    end
end

% Lift
Hlift = [H C/2;C'/2 zeros(size(C,2))];
if ops.kkt.dualpresolve.lplift
    XZ = sdpvar(nTOT);
    %    Hlift = [H C/2;C'/2 zeros(size(C,2))];
    % else
    %     if ops.kkt.dualbounds & ops.verbose & ~isempty(A) & ~isa(b,'double')
    %         disp('Dual bounds probably poor, could not lift (RHS is parameterized.)');
    %         disp('You might need to manually obtain bounds on the duals.)');
    %     end
end
constraints = [Lambda > 0,parametricDomain,F,L<xz<U];
constraints = [constraints,H*x + c + A'*Lambda==0];

if ops.kkt.dualpresolve.lplift
    if isa(b,'sdpvar')
        constraints = [constraints,-(trace(Hlift*XZ) + c0'*x) <= bmax'*Lambda];
        constraints = [constraints,-(trace(Hlift*XZ) + c0'*x) >= bmin'*Lambda];
    else
        constraints = [constraints,trace(Hlift*XZ) + c0'*x + b'*Lambda==0];
    end
    if nnz(Hlift)>0
        for i = 1:nTOT
            for j = i:nTOT
                % (xi-Li)(xj-Lj)>0
                constraints = [constraints, XZ(i,j)-L(i)*xz(j)-L(j)*xz(i)+L(i)*L(j)>0];
                % (xi-Li)(Uj-xj)>0
                constraints = [constraints, xz(i)*U(j)+L(i)*xz(j)-L(i)*U(j)-XZ(i,j)>0];
                % (xj-Lj)(Ui-xi)>0
                constraints = [constraints, xz(j)*U(i)+L(j)*xz(i)-L(j)*U(i)-XZ(j,i)>0];
                % (Uj-xj)(Ui-xi)>0
                constraints = [constraints, U(j)*U(i)-U(j)*xz(i)-xz(j)*U(i)+XZ(j,i)>0];
            end
        end
    end
end


%Z = sdpvar(nTOT + length(Lambda));Z(1:nTOT,1:nTOT)=XZ;
%mb = length(b);
%Hsdplift = [H C/2 zeros(n,mb);C'/2 zeros(size(C',1)) B'/2;zeros(mb,n) B/2 zeros(mb)]
%constraints = [constraints, trace(Hsdplift*Z)+c0'*x + b0'*Lambda == 0];


if ops.verbose
    disp(['*Computing ' num2str(length(Lambda)) ' dual bounds']);
end
ops2.verbose = 0;

dualUpper = inf(length(Lambda),1);
if 0%size(unique([A;-A],'rows'),1) == 2*size(A,1)
    for dualPass = 1:ops.kkt.dualpresolve.passes
        r = find(~isinf(dualUpper));
        if ~isempty(r)
            knownLambda = [Lambda(r) <= dualUpper(r)];
        else
            knownLambda = [];
        end
        
        sol =  solvesdp([knownLambda,constraints],-Lambda,ops2);
        for i = 1:m
            if ~sol.problem(i)
                selectsolution(i);
                dualUpper(i,1) = double(Lambda(i));
            end
        end
    end
else
    % always exploit disjoint constraints
    disjoints = zeros(length(b));
    for dualPass = 1:ops.kkt.dualpresolve.passes
        for i = 1:m
            if dualUpper(i)>0
                a = A(i,:);
                a = setdiff(findrows(A,-a),i);
                r = find(~isinf(dualUpper));
                if ~isempty(r)
                    knownLambda = [Lambda(r) < dualUpper(r)];
                else
                    knownLambda = [];
                end
                if isempty(a)                                        
                    sol = solvesdp([constraints,b(i)-A(i,:)*x==0,knownLambda,Lambda(find(disjoints(i,:)))==0],-Lambda(i),ops2);
                    if sol.problem == 12
                        sol = solvesdp([constraints,b(i)-A(i,:)*x==0,knownLambda,Lambda(find(disjoints(i,:)))==0],[],ops2);
                        if sol.problem == 1
                            dualUpper(i,1) = 0;
                        elseif sol.problem == 0
                            sol.problem = 2;
                        end
                    end
                else
                    sol = solvesdp([constraints,b(i)-A(i,:)*x == 0,Lambda(a)==0,knownLambda],-Lambda(i),ops2);
                    if sol.problem == 12
                        sol = solvesdp([constraints,b(i)-A(i,:)*x == 0,Lambda(a)==0,knownLambda],[],ops2);
                        if sol.problem == 1
                            dualUpper(i,1) = 0;
                        elseif sol.problem == 0
                            sol.problem = 2;
                        end
                    end
                end
                if sol.problem == 0
                    dualUpper(i,1) = double(Lambda(i));
                elseif sol.problem == 2
                    dualUpper(i,1) = inf;
                end
            end
        end
    end
end

if ops.verbose
    if all(isinf(dualUpper))
        disp('*Warning: All dual upper bounds are infinite');
    elseif any(isinf(dualUpper))
        disp('*Warning: Some of the dual upper bounds are infinite');
    else
        disp('*Success: All dual upper bounds are finite');
    end
end




